# TPK4186 - Advanced Tools for Performance Engineering
# StudentCohort.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of contents
# -----------------
# 1. Required modules
# 2. Students
# 3. Cohorts
# 4. Printer
# 5. Queries

# 1. Required modules
# -------------------
import sys
import math

# 2. Students
# -----------

def Student_New(identificationNumber):
  student = dict()
  student["IDENTIFICATION-NUMBER"] = identificationNumber
  student["COURSES"] = []
  return student
  
def Student_GetIdentificationNumber(student):
  return student.get("IDENTIFICATION-NUMBER", None)

def Student_IsIdentificationNumber(student, identificationNumber):
  return Student_GetIdentificationNumber(student)==identificationNumber

def Student_GetFamilyName(student):
  return student.get("FAMILY-NAME", None)

def Student_SetFamilyName(student, familyName):
  student["FAMILY-NAME"] = familyName

def Student_IsFamilyName(student, familyName):
  return Student_GetFamilyName(student)==familyName

def Student_GetFirstName(student):
  return student.get("FIRST-NAME", None)

def Student_SetFirstName(student, firstName):
  student["FIRST-NAME"] = firstName

def Student_IsFirstName(student, firstName):
  return Student_GetFirstName(student)==firstName

def Student_GetGender(student):
  return student.get("GENDER", None)

def Student_SetGender(student, familyName):
  student["GENDER"] = familyName

def Student_GetDayOfBirth(student):
    dateOfBirth = student.get("DATE-OF-BIRTH", None)
    if dateOfBirth==None:
      return None
    return dateOfBirth[0]

def Student_GetMonthOfBirth(student):
    dateOfBirth = student.get("DATE-OF-BIRTH", None)
    if dateOfBirth==None:
      return None
    return dateOfBirth[1]
 
def Student_GetYearOfBirth(student):
    dateOfBirth = student.get("DATE-OF-BIRTH", None)
    if dateOfBirth==None:
      return None
    return dateOfBirth[2]

def Student_SetDateOfBirth(student, day, month, year):
  student["DATE-OF-BIRTH"] = (day, month, year)

def Student_IsDayOfBirth(student, dayOfBirth):
  return Student_GetDayOfBirth(student)==dayOfBirth

def Student_IsMonthOfBirth(student, monthOfBirth):
  return Student_GetMonthOfBirth(student)==monthOfBirth

def Student_IsYearOfBirth(student, yearOfBirth):
  return Student_GetYearOfBirth(student)==yearOfBirth

def Student_GetEmailAddress(student):
  return student.get("EMAIL", None)

def Student_SetEmailAddress(student, email):
  student["EMAIL"] = email

def Student_IsRegistered(student, course):
  courses = student["COURSES"]
  return course in courses

def Student_Register(student, course):
  if Student_IsRegistered(student, course):
    return
  student["COURSES"].append(course)
  
def Student_CancelRegistration(student, course):
  if not Student_IsRegistered(student, course):
    return
  student["COURSES"].remove(course)
  
def Student_GetCourses(student):
  return student["COURSES"]

# 3. Cohorts
# ----------

def Cohort_New():
  return dict()

def Cohort_LookForStudent(cohort, identificationNumber):
  return cohort.get(identificationNumber, None)

def Cohort_GetStudents(cohort):
  return cohort.values()

def Cohort_NewStudent(cohort, identificationNumber):
  student = Cohort_LookForStudent(cohort, identificationNumber)
  if student!=None:
    return student
  student = Student_New(identificationNumber)
  cohort[identificationNumber] = student
  return student

def Cohort_RemoveStudent(cohort, student):
  if not student in cohort:
    return
  cohort.remove(student)

# 4. Printer
# ----------

def PrintStudent(student):
  print("Number:       \t" + str(Student_GetIdentificationNumber(student)))
  print("Family name:  \t" + Student_GetFamilyName(student))
  print("First name:   \t" + Student_GetFirstName(student))
  print("Date of birth:\t" + str(Student_GetDayOfBirth(student)) + "/" + str(Student_GetMonthOfBirth(student)) + "/" + str(Student_GetYearOfBirth(student)))
  courseList = ""
  for course in Student_GetCourses(student):
    if courseList!="":
      courseList += ", "
    courseList += course
  print("Courses:      \t" + courseList)

def PrintStudentList(studentList):
  for student in studentList:
    PrintStudent(student)
    print("")
  sys.stdout.flush()

# 5. Queries
# ----------

def Query_Select(studentList, test, datum):
  result = []
  for student in studentList:
    if test(student, datum):
      result.append(student)
  return result

def Query_Union(studentList1, studentList2):
  result = studentList1[:]
  for student in studentList2:
    if not student in studentList1:
      result.append(student)
  return result

def Query_Intersection(studentList1, studentList2):
  result =  []
  for student in studentList1:
    if student in studentList2:
      result.append(student)
  return result

def Query_Difference(studentList1, studentList2):
  result =  []
  for student in studentList1:
    if not student in studentList2:
      result.append(student)
  return result
